#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2018 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit Tests for `pdb_tocif`.
"""

import os
import sys
import unittest

from config import data_dir
from utils import OutputCapture


class TestTool(unittest.TestCase):
    """
    Generic class for testing tools.
    """

    def setUp(self):
        # Dynamically import the module
        name = 'pdbtools.pdb_tocif'
        self.module = __import__(name, fromlist=[''])

    def exec_module(self):
        """
        Execs module.
        """

        with OutputCapture() as output:
            try:
                self.module.main()
            except SystemExit as e:
                self.retcode = e.code

        self.stdout = output.stdout
        self.stderr = output.stderr

        return

    def test_single_model_run_iterable(self):
        """$ pdb_tocif.run(iterable)"""
        from pdbtools import pdb_tocif

        fpath = os.path.join(data_dir, 'dummy.pdb')
        with open(fpath, 'r') as fin:
            lines = fin.readlines()

        newlines = list(pdb_tocif.run(lines))

        # Check no of records
        n_ATOM = sum(1 for l in newlines if l.startswith('ATOM'))
        n_HETATM = sum(1 for l in newlines if l.startswith('HETATM'))
        n_coord = n_ATOM + n_HETATM
        self.assertEqual(n_ATOM, 176)
        self.assertEqual(n_HETATM, 9)
        self.assertEqual(n_coord, 185)

        # check name
        self.assertEqual(newlines[2], 'data_cell\n')

    def test_single_model_run_iterable_with_name(self):
        """$ pdb_tocif.run(iterable)"""
        from pdbtools import pdb_tocif

        fpath = os.path.join(data_dir, 'dummy.pdb')
        with open(fpath, 'r') as fin:
            lines = fin.readlines()

        newlines = list(pdb_tocif.run(lines, outname='newcif'))

        # Check no of records
        n_ATOM = sum(1 for l in newlines if l.startswith('ATOM'))
        n_HETATM = sum(1 for l in newlines if l.startswith('HETATM'))
        n_coord = n_ATOM + n_HETATM
        self.assertEqual(n_ATOM, 176)
        self.assertEqual(n_HETATM, 9)
        self.assertEqual(n_coord, 185)

        # check name
        self.assertEqual(newlines[2], 'data_newcif\n')

    def test_single_model_run_fhandler(self):
        """$ pdb_tocif.run(fhandler)"""
        from pdbtools import pdb_tocif

        fpath = os.path.join(data_dir, 'dummy.pdb')
        with open(fpath, 'r') as fin:
            newlines = list(pdb_tocif.run(fin))

        # Check no of records
        n_ATOM = sum(1 for l in newlines if l.startswith('ATOM'))
        n_HETATM = sum(1 for l in newlines if l.startswith('HETATM'))
        n_coord = n_ATOM + n_HETATM
        self.assertEqual(n_ATOM, 176)
        self.assertEqual(n_HETATM, 9)
        self.assertEqual(n_coord, 185)

        # check name
        self.assertEqual(newlines[2], 'data_dummy\n')

    def test_single_model_run_fhandler_name(self):
        """$ pdb_tocif.run(fhandler)"""
        from pdbtools import pdb_tocif

        fpath = os.path.join(data_dir, 'dummy.pdb')
        with open(fpath, 'r') as fin:
            newlines = list(pdb_tocif.run(fin, outname='newname'))

        # Check no of records
        n_ATOM = sum(1 for l in newlines if l.startswith('ATOM'))
        n_HETATM = sum(1 for l in newlines if l.startswith('HETATM'))
        n_coord = n_ATOM + n_HETATM
        self.assertEqual(n_ATOM, 176)
        self.assertEqual(n_HETATM, 9)
        self.assertEqual(n_coord, 185)

        # check name
        self.assertEqual(newlines[2], 'data_newname\n')

    def test_single_model(self):
        """$ pdb_tocif data/dummy.pdb"""

        fpath = os.path.join(data_dir, 'dummy.pdb')
        sys.argv = ['', fpath]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)
        self.assertEqual(len(self.stdout), 212)
        self.assertEqual(len(self.stderr), 0)

        # Check no of records
        n_ATOM = len([l for l in self.stdout if l.startswith('ATOM')])
        n_HETATM = len([l for l in self.stdout if l.startswith('HETATM')])
        n_coord = n_ATOM + n_HETATM
        self.assertEqual(n_ATOM, 176)
        self.assertEqual(n_HETATM, 9)
        self.assertEqual(n_coord, 185)

    def test_multi_model(self):
        """$ pdb_tocif data/ensemble_OK.pdb"""

        fpath = os.path.join(data_dir, 'ensemble_OK.pdb')
        sys.argv = ['', fpath]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)
        self.assertEqual(len(self.stdout), 31)
        self.assertEqual(len(self.stderr), 0)

        # Check no of records
        n_ATOM = len([l for l in self.stdout if l.startswith('ATOM')])
        n_HETATM = len([l for l in self.stdout if l.startswith('HETATM')])
        n_coord = n_ATOM + n_HETATM
        self.assertEqual(n_ATOM, 4)
        self.assertEqual(n_HETATM, 0)
        self.assertEqual(n_coord, 4)

        # Check model numbers
        records = (('ATOM', 'HETATM'))
        models = [l.split()[-1] for l in self.stdout if l.startswith(records)]
        self.assertEqual(models, ['1', '1', '2', '2'])

        # Check number of fields
        atom_lines = [l for l in self.stdout if l.startswith(records)]
        n_fields = list(set(map(lambda x: len(x.split()), atom_lines)))
        self.assertEqual(n_fields, [21])

    def test_file_not_found(self):
        """$ pdb_tocif not_existing.pdb"""

        # Error (file not found)
        afile = os.path.join(data_dir, 'not_existing.pdb')
        sys.argv = ['', afile]

        # Execute the script
        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:22],
                         "ERROR!! File not found")

    @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    def test_helptext(self):
        """$ pdb_tocif"""

        sys.argv = ['']

        # Execute the script
        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr, self.module.__doc__.split("\n")[:-1])

    def test_invalid_option(self):
        """$ pdb_tocif -A data/dummy.pdb"""

        sys.argv = ['', '-A', os.path.join(data_dir, 'dummy.pdb')]

        # Execute the script
        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:36],
                         "ERROR!! Script takes 1 argument, not")


if __name__ == '__main__':
    from config import test_dir

    mpath = os.path.abspath(os.path.join(test_dir, '..'))
    sys.path.insert(0, mpath)  # so we load dev files before  any installation

    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2018 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit Tests for `pdb_fixinsert`.
"""

import os
import sys
import unittest
import warnings

from config import data_dir
from utils import OutputCapture


class TestTool(unittest.TestCase):
    """
    Generic class for testing tools.
    """

    def setUp(self):
        # Dynamically import the module
        name = 'pdbtools.pdb_delinsertion'
        self.module = __import__(name, fromlist=[''])

    def exec_module(self):
        """
        Execs module.
        """

        with OutputCapture() as output:
            try:
                with warnings.catch_warnings():
                    warnings.simplefilter("ignore")
                    self.module.main()
            except SystemExit as e:
                self.retcode = e.code

        self.stdout = output.stdout
        self.stderr = output.stderr

        return

    def test_default(self):
        """$ pdb_fixinsert data/dummy_insertions.pdb"""

        # Simulate input
        # pdb_fixinsert dummy_insertions.pdb
        sys.argv = ['', os.path.join(data_dir, 'dummy_insertions.pdb')]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        self.assertEqual(len(self.stdout), 255)  # no lines deleted
        self.assertEqual(len(self.stderr), 0)  # no errors

        # Check if we do not have any insertions
        records = (('ATOM', 'HETATM', 'TER'))
        icodes = set(l[26] for l in self.stdout if l.startswith(records))
        self.assertEqual(icodes, set(' '))

        # Check numbering was corrected
        resid = [int(l[22:26]) for l in self.stdout if l.startswith(records)]
        expected = [
            4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
            4, 4, 4, 5, 5, 5, 5, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
            8, 8, 8, 8, 8, 8, 8, 8, 8, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 7, 7, 7,
            7, 7, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 9, 9, 9, 9, 9, 9,
            9, 9, 9, 9, 9, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2,
            2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
            3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5,
            5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
            2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, -1, -1, -1, -1, -1, -1,
            -1, -1, -1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
            2, 301, 302, 303, 301, 301, 302, 303, 304, 305
        ]
        self.assertEqual(resid, expected)

    def test_select_insertion(self):
        """$ pdb_fixinsert -A1 data/dummy_insertions.pdb"""

        sys.argv = ['', '-A1', os.path.join(data_dir, 'dummy_insertions.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 0)
        self.assertEqual(len(self.stdout), 255)
        self.assertEqual(len(self.stderr), 0)

        records = (('ATOM', 'HETATM', 'TER'))

        # Check the insertions were deleted specifically
        for line in self.stdout:
            if line.startswith(records):
                icode = line[26]
                resid = int(line[22:26])
                if line[17:26] == 'ARG B   4':
                    self.assertIn(icode, ('A', 'B'))
                    self.assertEqual(resid, 4)
                elif line[17:26] == 'GLY B   4':
                    self.assertEqual(icode, ' ')
                    self.assertEqual(resid, 4)
                elif line[17:26] == 'ASN A   1':
                    self.assertEqual(icode, ' ')
                    self.assertEqual(resid, 1)
                elif line[17:26] == 'GLY A   1':
                    self.assertEqual(icode, ' ')
                    self.assertEqual(resid, 2)  # should have renumbered

        # Check numbering was corrected overall
        resid = [int(l[22:26]) for l in self.stdout if l.startswith(records)]
        expected = [
            4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
            4, 4, 4, 5, 5, 5, 5, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
            8, 8, 8, 8, 8, 8, 8, 8, 8, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 7, 7, 7,
            7, 7, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 9, 9, 9, 9, 9, 9,
            9, 9, 9, 9, 9, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2,
            2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
            3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5,
            5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
            2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, -1, -1, -1, -1, -1, -1,
            -1, -1, -1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
            2, 301, 302, 303, 301, 301, 302, 303, 304, 305
        ]

        self.assertEqual(resid, expected)

    def test_file_not_found(self):
        """$ pdb_fixinsert not_existing.pdb"""

        afile = os.path.join(data_dir, 'not_existing.pdb')
        sys.argv = ['', afile]

        self.exec_module()

        self.assertEqual(self.retcode, 1)  # exit code is 1 (error)
        self.assertEqual(len(self.stdout), 0)  # nothing written to stdout
        self.assertEqual(self.stderr[0][:22],
                         "ERROR!! File not found")  # proper error message

    @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    def test_file_missing(self):
        """$ pdb_fixinsert -A89"""

        sys.argv = ['', '-A89']

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)  # no output
        self.assertEqual(self.stderr[0],
                         "ERROR!! No data to process!")

    # @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    # def test_helptext(self):
    #     """$ pdb_fixinsert"""

    #     sys.argv = ['']

    #     self.exec_module()

    #     self.assertEqual(self.retcode, 1)  # ensure the program exited gracefully.
    #     self.assertEqual(len(self.stdout), 0)  # no output
    #     self.assertEqual(self.stderr, self.module.__doc__.split("\n")[:-1])

    def test_invalid_option(self):
        """$ pdb_fixinsert -A data/dummy_insertions.pdb"""

        sys.argv = ['', '-A', os.path.join(data_dir, 'dummy_insertions.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:27],
                         "ERROR!! Option invalid: 'A'")

    def test_invalid_option_2(self):
        """$ pdb_fixinsert -12A data/dummy_insertions.pdb"""

        sys.argv = ['', '-12A', os.path.join(data_dir, 'dummy_insertions.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:39],
                         "ERROR!! Option invalid: '12A'")

    def test_not_an_option(self):
        """$ pdb_fixinsert 20 data/dummy_insertions.pdb"""

        sys.argv = ['', '20', os.path.join(data_dir, 'dummy_insertions.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0],
                         "ERROR! First argument is not an option: '20'")


if __name__ == '__main__':
    from config import test_dir

    mpath = os.path.abspath(os.path.join(test_dir, '..'))
    sys.path.insert(0, mpath)  # so we load dev files before  any installation

    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 1118 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit Tests for `pdb_shiftres`.
"""

import os
import sys
import unittest

from config import data_dir
from utils import OutputCapture


class TestTool(unittest.TestCase):
    """
    Generic class for testing tools.
    """

    def setUp(self):
        # Dynamically import the module
        name = 'pdbtools.pdb_shiftres'
        self.module = __import__(name, fromlist=[''])

    def exec_module(self):
        """
        Execs module.
        """

        with OutputCapture() as output:
            try:
                self.module.main()
            except SystemExit as e:
                self.retcode = e.code

        self.stdout = output.stdout
        self.stderr = output.stderr

        return

    def test_default(self):
        """$ pdb_shiftres data/dummy.pdb"""

        # Simulate input
        # pdb_shiftres dummy.pdb
        sys.argv = ['', os.path.join(data_dir, 'dummy.pdb')]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        self.assertEqual(len(self.stdout), 204)  # no lines deleted
        self.assertEqual(len(self.stderr), 0)  # no errors

        records = (('ATOM', 'HETATM'))
        resid_list = [int(l[22:26]) for l in self.stdout
                      if l.startswith(records)]

        expected = [4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
                    4, 4, 4, 4, 4, 4, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
                    6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 1, 1, 1, 1, 1,
                    1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
                    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3,
                    3, 3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
                    5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 2, 2, 2, 2, 2, 2, 2,
                    2, 2, 2, 2, 2, 2, 2, 2, -1, -1, -1, -1, -1, -1, -1, -1, 2,
                    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
                    301, 302, 303, 301, 301, 302, 303, 304, 305]

        self.assertEqual(resid_list, expected)

    def test_single_option_1(self):
        """$ pdb_shiftres -10 data/dummy.pdb"""

        # Simulate input
        # pdb_shiftres dummy.pdb
        sys.argv = ['', '-10', os.path.join(data_dir, 'dummy.pdb')]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        self.assertEqual(len(self.stdout), 204)  # no lines deleted
        self.assertEqual(len(self.stderr), 0)  # no errors

        records = (('ATOM', 'HETATM'))
        resid_list = [int(l[22:26]) for l in self.stdout
                      if l.startswith(records)]

        expected = [14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,
                    14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 16, 16, 16, 16,
                    16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 17, 17, 17, 17,
                    17, 17, 17, 17, 17, 17, 11, 11, 11, 11, 11, 11, 11, 11, 11,
                    11, 11, 11, 11, 11, 11, 11, 11, 12, 12, 12, 12, 12, 12, 12,
                    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
                    12, 12, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13,
                    13, 13, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15,
                    15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 12, 12,
                    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 9, 9, 9,
                    9, 9, 9, 9, 9, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
                    12, 12, 12, 12, 12, 12, 12, 12, 12, 311, 312, 313, 311, 311,
                    312, 313, 314, 315]

        self.assertEqual(resid_list, expected)

    def test_single_option_2(self):
        """$ pdb_shiftres --10 data/dummy.pdb"""

        # Simulate input
        # pdb_shiftres dummy.pdb
        sys.argv = ['', '--10', os.path.join(data_dir, 'dummy.pdb')]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        self.assertEqual(len(self.stdout), 204)  # no lines deleted
        self.assertEqual(len(self.stderr), 0)  # no errors

        records = (('ATOM', 'HETATM'))
        resid_list = [int(l[22:26]) for l in self.stdout
                      if l.startswith(records)]

        expected = [-6, -6, -6, -6, -6, -6, -6, -6, -6, -6, -6, -6, -6, -6, -6,
                    -6, -6, -6, -6, -6, -6, -6, -6, -6, -6, -6, -4, -4, -4, -4,
                    -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -3, -3, -3, -3,
                    -3, -3, -3, -3, -3, -3, -9, -9, -9, -9, -9, -9, -9, -9, -9,
                    -9, -9, -9, -9, -9, -9, -9, -9, -8, -8, -8, -8, -8, -8, -8,
                    -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8,
                    -8, -8, -7, -7, -7, -7, -7, -7, -7, -7, -7, -7, -7, -7, -7,
                    -7, -7, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5,
                    -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -8, -8,
                    -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -11,
                    -11, -11, -11, -11, -11, -11, -11, -8, -8, -8, -8, -8, -8,
                    -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, 291,
                    292, 293, 291, 291, 292, 293, 294, 295]

        self.assertEqual(resid_list, expected)

    def test_too_many_residues(self):
        """$ pdb_shiftres -9998 data/dummy.pdb"""

        sys.argv = ['', '-9998', os.path.join(data_dir, 'dummy.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(len(self.stderr), 1)

        self.assertEqual(self.stderr[0][:22],
                         "Cannot set residue num")  # proper error message

    def test_file_not_found(self):
        """$ pdb_shiftres -10 not_existing.pdb"""

        afile = os.path.join(data_dir, 'not_existing.pdb')
        sys.argv = ['', '-10', afile]

        self.exec_module()

        self.assertEqual(self.retcode, 1)  # exit code is 1 (error)
        self.assertEqual(len(self.stdout), 0)  # nothing written to stdout
        self.assertEqual(self.stderr[0][:22],
                         "ERROR!! File not found")  # proper error message

    @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    def test_file_missing(self):
        """$ pdb_shiftres -10"""

        sys.argv = ['', '-10']

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)  # no output
        self.assertEqual(self.stderr[0],
                         "ERROR!! No data to process!")

    @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    def test_helptext(self):
        """$ pdb_shiftres"""

        sys.argv = ['']

        self.exec_module()

        self.assertEqual(self.retcode, 1)  # ensure the program exited gracefully.
        self.assertEqual(len(self.stdout), 0)  # no output
        self.assertEqual(self.stderr, self.module.__doc__.split("\n")[:-1])

    def test_invalid_option(self):
        """$ pdb_shiftres -A data/dummy.pdb"""

        sys.argv = ['', '-A', os.path.join(data_dir, 'dummy.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:36],
                         "ERROR!! You provided an invalid numb")

    def test_not_an_option(self):
        """$ pdb_shiftres 11 data/dummy.pdb"""

        sys.argv = ['', '11', os.path.join(data_dir, 'dummy.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0],
                         "ERROR! First argument is not an option: '11'")


if __name__ == '__main__':
    from config import test_dir

    mpath = os.path.abspath(os.path.join(test_dir, '..'))
    sys.path.insert(0, mpath)  # so we load dev files before  any installation

    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2018 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Utility classes/functions for the Tests.
"""

try:
    from cStringIO import StringIO
except ImportError:  # Py 3.x?
    from io import StringIO

import sys


class OutputCapture(object):
    """Context manager to capture output usually redirected to stdout.

    Use as:
    >>> with OutputCapture() as output:
    >>> ....run_stuff()
    >>> print(output)  # list with lines
    """

    def __enter__(self):
        self.stdout = []
        self.stderr = []

        self._stdout = sys.stdout
        self._stderr = sys.stderr
        sys.stdout = self._stringout = StringIO()
        sys.stderr = self._stringerr = StringIO()
        return self

    def __exit__(self, *args):

        self.stdout.extend(self._stringout.getvalue().splitlines())
        self.stderr.extend(self._stringerr.getvalue().splitlines())
        del self._stringout    # free up some memory
        del self._stringerr    # free up some memory
        sys.stdout = self._stdout
        sys.stderr = self._stderr



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2018 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit Tests for all pdb-tools scripts
"""

import os
import sys
import unittest

from .config import test_dir


def test_all():
    mpath = os.path.abspath(os.path.join(test_dir, '..'))
    sys.path.insert(0, mpath)  # so we load dev files before  any installation

    loader = unittest.defaultTestLoader

    tpath = os.path.join(mpath, 'tests')
    suite = loader.discover(tpath)

    return suite



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2018 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit Tests for `pdb_fromcif`.
"""

import os
import sys
import unittest

from config import data_dir
from utils import OutputCapture


class TestTool(unittest.TestCase):
    """
    Generic class for testing tools.
    """

    def setUp(self):
        # Dynamically import the module
        name = "pdbtools.pdb_fromcif"
        self.module = __import__(name, fromlist=[""])

    def exec_module(self):
        """
        Execs module.
        """

        with OutputCapture() as output:
            try:
                self.module.main()
            except SystemExit as e:
                self.retcode = e.code

        self.stdout = output.stdout
        self.stderr = output.stderr

        return

    def test_conversion(self):
        """$ pdb_fromcif data/ensemble_OK.cif"""

        fpath = os.path.join(data_dir, "ensemble_OK.cif")
        sys.argv = ["", fpath]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)
        self.assertEqual(len(self.stdout), 9)
        self.assertEqual(len(self.stderr), 0)

        # Check order of records
        expected = [
            "MODEL ",
            "ATOM  ",
            "ATOM  ",
            "ENDMDL",
            "MODEL ",
            "ATOM  ",
            "ATOM  ",
            "ENDMDL",
            "END   ",
        ]
        records = [l[:6] for l in self.stdout]
        self.assertEqual(records, expected)

    def test_file_not_found(self):
        """$ pdb_fromcif not_existing.pdb"""

        # Error (file not found)
        afile = os.path.join(data_dir, "not_existing.pdb")
        sys.argv = ["", afile]

        # Execute the script
        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:22], "ERROR!! File not found")

    @unittest.skipIf(os.getenv("SKIP_TTY_TESTS"), "skip on GHA - no TTY")
    def test_helptext(self):
        """$ pdb_fromcif"""

        sys.argv = [""]

        # Execute the script
        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr, self.module.__doc__.split("\n")[:-1])

    def test_invalid_option(self):
        """$ pdb_fromcif -A data/dummy.cif"""

        sys.argv = ["", "-A", os.path.join(data_dir, "dummy.cif")]

        # Execute the script
        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:43], "ERROR! First argument is not a valid option")

    def test_valid_option(self):
        """$ pdb_fromcif -h36 data/dummy-h36.cif"""

        sys.argv = ["", "-h36", os.path.join(data_dir, "dummy-h36.cif")]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)
        self.assertEqual(len(self.stdout), 28)
        self.assertEqual(len(self.stderr), 0)

    def test_h36_encoding(self):
        """Tests that atom serials > 99999 are encoded in hybrid-36 format"""

        def _cif_lines(n_atoms):
            yield "loop_\n"
            for field in [
                "group_PDB", "id", "type_symbol", "label_atom_id",
                "label_alt_id", "label_comp_id", "label_asym_id",
                "label_entity_id", "label_seq_id", "pdbx_PDB_ins_code",
                "Cartn_x", "Cartn_y", "Cartn_z", "occupancy",
                "B_iso_or_equiv", "pdbx_formal_charge", "auth_asym_id",
                "pdbx_PDB_model_num",
            ]:
                yield "_atom_site.{}\n".format(field)
            for i in range(1, n_atoms + 1):
                yield (
                    "ATOM {} N N . ALA A . 1 ? "
                    "0.000 0.000 0.000 1.00 15.00 0 A 1\n"
                ).format(i)

        result = list(self.module.run(_cif_lines(100001), h36=True))

        # 100001 ATOM lines + END
        self.assertEqual(len(result), 100002)
        # last decimal serial
        self.assertEqual(result[99998][6:11], "99999")
        # serial 100000 encodes as "A0000" in hybrid-36
        self.assertEqual(result[99999][6:11], "A0000")
        # serial 100001 encodes as "A0001" in hybrid-36
        self.assertEqual(result[100000][6:11], "A0001")

    def test_missing_charge(self):
        """$ pdb_fromcif data/af3_output.cif"""
        fpath = os.path.join(data_dir, "af3_output.cif")
        sys.argv = ["", fpath]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)
        self.assertEqual(len(self.stdout), 3928)
        self.assertEqual(len(self.stderr), 0)


if __name__ == "__main__":
    from config import test_dir

    mpath = os.path.abspath(os.path.join(test_dir, ".."))
    sys.path.insert(0, mpath)  # so we load dev files before  any installation

    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2018 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# === HYBRID 36 FUNCTIONS TAKEN FROM CCTBX ===
# Copyright (c) 2006-2026, The Regents of the University of California,
# through Lawrence Berkeley National Laboratory. All rights reserved.
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided the above copyright notice and
# this paragraph are included. Full license text available at
# [https://github.com/cctbx/cctbx_project/blob/releases/2023.1/LICENSE.txt].
#

"""
Modifies the file to adhere (as much as possible) to the format specifications.

Expects a sorted file - REMARK/ATOM/HETATM/END - so use pdb_sort in case you are
not sure.

This includes:
    - Adding TER statements after chain breaks/changes
    - Truncating/Padding all lines to 80 characters
    - Adds END statement at the end of the file

Will remove all original TER/END statements from the file.

Usage:
    python pdb_tidy.py [-strict] [-h36] <pdb file>

Options:
    -h36: allows for hybrid36 output format for encoding >99999 atoms in the PDB file

Example:
    python pdb_tidy.py 1CTF.pdb
    python pdb_tidy.py -strict 1CTF.pdb  # does not add TER on chain breaks
    python pdb_tidy.py -h36 1CTF.pdb  # allows for a number of atom >99999 by using hybrid_36 format

This program is part of the `pdb-tools` suite of utilities and should not be
distributed isolatedly. The `pdb-tools` were created to quickly manipulate PDB
files using the terminal, and can be used sequentially, with one tool streaming
data to another. They are based on old FORTRAN77 code that was taking too much
effort to maintain and compile. RIP.
"""

import os
import sys
import re
import textwrap

__author__ = "Joao Rodrigues"
__email__ = "j.p.g.l.m.rodrigues@gmail.com"


digits_upper = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
digits_lower = digits_upper.lower()
digits_upper_values = dict([pair for pair in zip(digits_upper, range(36))])
digits_lower_values = dict([pair for pair in zip(digits_lower, range(36))])


# Copied from CCTBX
def encode_pure(digits, value):
    "encodes value using the given digits"
    assert value >= 0
    if (value == 0):
        return digits[0]
    n = len(digits)
    result = []
    while (value != 0):
        rest = value // n
        result.append(digits[value - rest * n])
        value = rest
    result.reverse()
    return "".join(result)


# Copied from CCTBX
def hy36encode(width, value):
    "encodes value as base-10/upper-case base-36/lower-case base-36 hybrid"
    i = value
    if (i >= 1 - 10**(width - 1)):
        if (i < 10**width):
            return ("%%%dd" % width) % i
        i -= 10**width
        if (i < 26 * 36**(width - 1)):
            i += 10 * 36**(width - 1)
            return encode_pure(digits_upper, i)
        i -= 26 * 36**(width - 1)
        if (i < 26 * 36**(width - 1)):
            i += 10 * 36**(width - 1)
            return encode_pure(digits_lower, i)
    raise ValueError("value out of range.")


# Copied from CCTBX
def decode_pure(digits_values, s):
    "decodes the string s using the digit, value associations for each character"
    result = 0
    n = len(digits_values)
    for c in s:
        result *= n
        result += digits_values[c]
    return result


# Copied from CCTBX
def hy36decode(width, s):
    "decodes base-10/upper-case base-36/lower-case base-36 hybrid"
    if (len(s) == width):
        f = s[0]
        if (f == "-" or f == " " or f.isdigit()):
            try:
                return int(s)
            except ValueError:
                pass
            if (s == " " * width):
                return 0
        elif (f in digits_upper_values):
            try:
                return decode_pure(
                    digits_values=digits_upper_values, s=s) - 10 * 36**(width - 1) + 10**width
            except KeyError:
                pass
        elif (f in digits_lower_values):
            try:
                return decode_pure(
                    digits_values=digits_lower_values, s=s) + 16 * 36**(width - 1) + 10**width
            except KeyError:
                pass
    raise ValueError("invalid number literal.")


def check_input(args):
    """Checks whether to read from stdin/file and validates user input/options.
    """

    # Defaults
    option = False
    h36option = False
    fh = sys.stdin  # file handle

    if not len(args):
        # Reading from pipe with default option
        if sys.stdin.isatty():
            sys.stderr.write(__doc__)
            sys.exit(1)

    elif len(args) == 1:
        # One of two options: option & Pipe OR file & default option
        if args[0] == '-strict':
            option = True
            if sys.stdin.isatty():  # ensure the PDB data is streamed in
                emsg = 'ERROR!! No data to process!\n'
                sys.stderr.write(emsg)
                sys.stderr.write(__doc__)
                sys.exit(1)
        elif args[0] == '-h36':
            h36option = True
            if sys.stdin.isatty():  # ensure the PDB data is streamed in
                emsg = 'ERROR!! No data to process!\n'
                sys.stderr.write(emsg)
                sys.stderr.write(__doc__)
                sys.exit(1)
        else:
            if not os.path.isfile(args[0]):
                emsg = 'ERROR!! File not found or not readable: \'{}\'\n'
                sys.stderr.write(emsg.format(args[0]))
                sys.stderr.write(__doc__)
                sys.exit(1)

            fh = open(args[0], 'r')

    elif len(args) == 2:
        # Two options: option & File or two options & Pipe
        if not args[0].startswith('-'):
            emsg = 'ERROR! First argument is not a valid option: \'{}\'\n'
            sys.stderr.write(emsg.format(args[0]))
            sys.stderr.write(__doc__)
            sys.exit(1)

        if args[0].startswith('-') and not args[1].startswith('-'):
            if args[0] == '-strict':
                option = True
            elif args[0] == '-h36':
                h36option = True
            else:
                emsg = 'ERROR! First argument is not a valid option: \'{}\'\n'
                sys.stderr.write(emsg.format(args[0]))
                sys.stderr.write(__doc__)
                sys.exit(1)

            if not os.path.isfile(args[1]):
                emsg = 'ERROR!! File not found or not readable: \'{}\'\n'
                sys.stderr.write(emsg.format(args[1]))
                sys.stderr.write(__doc__)
                sys.exit(1)
            fh = open(args[1], 'r')

        elif args[0].startswith('-') and args[1].startswith('-'):
            if args[0] == args[1]:
                emsg = 'ERROR! The two arguments are the same: \'{}\'\n'
                sys.stderr.write(emsg.format(args[0]))
                sys.stderr.write(__doc__)
                sys.exit(1)
            else:
                if '-strict' in args[:2]:
                    option = True
                if '-h36' in args[:2]:
                    h36option = True
                if sys.stdin.isatty():  # ensure the PDB data is streamed in
                    emsg = 'ERROR!! No data to process!\n'
                    sys.stderr.write(emsg)
                    sys.stderr.write(__doc__)
                    sys.exit(1)

    elif len(args) == 3:
        # One option: two options & File
        if not args[0].startswith('-') and args[1].startswith('-'):
            emsg = 'ERROR! First two argument are not a valid option: \'{}\'\n'
            sys.stderr.write(emsg.format(args[0]))
            sys.stderr.write(__doc__)
            sys.exit(1)
        else:
            if args[0] == '-strict' and args[1] == '-h36':
                option = True
                h36option = True
            elif args[0] == '-h36' and args[1] == '-strict':
                option = True
                h36option = True
            else:
                emsg = 'ERROR! Arguments are not valid options: \'{}\' \'{}\'\n'
                sys.stderr.write(emsg.format(args[0], args[1]))
                sys.stderr.write(__doc__)
                sys.exit(1)
            if not os.path.isfile(args[2]):
                emsg = 'ERROR!! File not found or not readable: \'{}\'\n'
                sys.stderr.write(emsg.format(args[1]))
                sys.stderr.write(__doc__)
                sys.exit(1)
            fh = open(args[2], 'r')

    else:  # Whatever ...
        sys.stderr.write(__doc__)
        sys.exit(1)

    return (fh, option, h36option)


def run(fhandle, strict=False, h36=False):
    """
    Add TER/END statements and truncates/pads all lines to 80 characters.

    This function is a generator.

    Parameters
    ----------
    fhandle : a line-by-line iterator of the original PDB file.

    strict : bool
        If True, does not add TER statements at intra-chain breaks.

    h36 : bool
        If True, allows for hybrid_36 format enabling the encoding for >99999 atoms.

    Yields
    ------
    str (line-by-line)
        The modified (or not) PDB line.
    """
    not_strict = not strict
    not_h36 = not h36
    fhandle = iter(fhandle)

    def make_TER(prev_line):
        """Creates a TER statement based on the last ATOM/HETATM line.
        """

        # Add last TER statement
        try:
            nserial = int(prev_line[6:11])
        except ValueError:
            nserial = hy36decode(5, prev_line[6:11])

        serial = nserial + 1
        rname = prev_line[17:20]
        chain = prev_line[21]
        resid = prev_line[22:26]
        icode = prev_line[26]

        return fmt_TER.format(serial, rname, chain, resid, icode)

    # TER     606      LEU A  75
    fmt_TER = "TER  {:>6d}      {:3s} {:1s}{:>4s}{:1s}" + " " * 53 + "\n"

    records = ('ATOM', 'HETATM')
    ignored = ('TER', 'END', 'CONECT', 'MASTER', 'ENDMDL')
    # Iterate up to the first ATOM/HETATM line
    prev_line = None
    num_models = 1
    in_model = False
    for line in fhandle:

        line = line.strip()  # We will pad/add \n later to make uniform

        if line.startswith('MODEL'):
            line = "MODEL " + "    " + str(num_models).rjust(4)
            num_models += 1
            in_model = True

        if line.startswith(ignored):  # to avoid matching END _and_ ENDMDL
            continue

        # Check line length, wrapping and padding as necessary
        # preserve the line prefix for wrapping
        prefix = re.match(r"\S+\s*", line).group(0)
        content = line[len(prefix):].lstrip()

        line = "".join(
            f"{prefix}{part:<{80 - len(prefix)}}\n"
            for part in textwrap.wrap(content, width=80 - len(prefix)))

        yield line

        if line.startswith(records):
            prev_line = line
            break

    # Now go through all the remaining lines
    atom_section = False
    serial_offset = 0  # To offset after adding TER records
    for line in fhandle:

        line = line.strip()

        if line.startswith(ignored):
            continue

        # Treat ATOM/HETATM differently
        #   - no TER in HETATM
        if line.startswith('ATOM'):

            is_gap = (int(line[22:26]) - int(prev_line[22:26])) > 1
            if atom_section and (line[21] != prev_line[21] or (not_strict and is_gap)):
                serial_offset += 1  # account for TER statement
                yield make_TER(prev_line)

            try:
                nserial = int(line[6:11])
            except ValueError:
                nserial = hy36decode(5, line[6:11])

            if not_h36 and nserial > 99999:
                emsg = 'ERROR!! Structure contains more than 99.999 atoms.\n'
                sys.stderr.write(emsg)
                sys.stderr.write(__doc__)
                sys.exit(1)

            serial = nserial + serial_offset
            if serial > 99999:
                if h36:
                    line = line[:6] + hy36encode(5, serial) + line[11:]
                else:
                    emsg = 'ERROR!! Structure contains more than 99.999 atoms.\n'
                    sys.stderr.write(emsg)
                    sys.stderr.write(__doc__)
                    sys.exit(1)
            else:
                line = line[:6] + str(serial).rjust(5) + line[11:]
            prev_line = line
            atom_section = True

        elif line.startswith('HETATM'):
            if atom_section:
                atom_section = False
                serial_offset += 1  # account for TER statement
                yield make_TER(prev_line)

            try:
                nserial = int(line[6:11])
            except ValueError:
                nserial = hy36decode(5, line[6:11])

            serial = nserial + serial_offset
            if serial > 99999 and not h36:
                emsg = 'Cannot represent atom serial number above 99999 without -h36\n'
                sys.stderr.write(emsg)
                sys.exit(1)
            elif serial > 99999:
                line = line[:6] + hy36encode(5, serial) + line[11:]
            else:
                line = line[:6] + str(serial).rjust(5) + line[11:]
            prev_line = line

        elif line.startswith('ANISOU'):
            # Copy serial directly from the preceding ATOM/HETATM (already formatted)
            line = line[:6] + prev_line[6:11] + line[11:]

        else:
            if atom_section:
                atom_section = False
                yield make_TER(prev_line)
            if in_model:
                yield "{:<80}\n".format("ENDMDL")
                in_model = False

            if line.startswith('MODEL'):
                line = "MODEL " + "    " + str(num_models).rjust(4)
                num_models += 1
                in_model = True
                serial_offset = 0

        # Check line length
        line = "{:<80}\n".format(line)

        yield line

    else:
        if atom_section:
            # Add last TER statement
            atom_section = False
            yield make_TER(prev_line)
        if in_model:
            yield "{:<80}\n".format("ENDMDL")
            in_model = False

    # Add END statement
    yield "{:<80}\n".format("END")


tidy_pdbfile = run


def main():
    # Check Input
    pdbfh, strict, h36 = check_input(sys.argv[1:])

    # Do the job
    new_pdb = run(pdbfh, strict, h36)

    try:
        _buffer = []
        _buffer_size = 5000  # write N lines at a time
        for lineno, line in enumerate(new_pdb):
            if not (lineno % _buffer_size):
                sys.stdout.write(''.join(_buffer))
                _buffer = []
            _buffer.append(line)

        sys.stdout.write(''.join(_buffer))
        sys.stdout.flush()
    except IOError:
        # This is here to catch Broken Pipes
        # for example to use 'head' or 'tail' without
        # the error message showing up
        pass

    # last line of the script
    # We can close it even if it is sys.stdin
    pdbfh.close()
    sys.exit(0)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2018 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit Tests for `pdb_tidy`.
"""

try:
    from StringIO import StringIO  # python 2.7
except ImportError:
    from io import StringIO  # python 3.x

import os
import sys
import unittest

from config import data_dir
from utils import OutputCapture


class TestTool(unittest.TestCase):
    """
    Generic class for testing tools.
    """

    def setUp(self):
        # Dynamically import the module
        name = 'pdbtools.pdb_tidy'
        self.module = __import__(name, fromlist=[''])

    def exec_module(self, stdin=None):
        """
        Execs module.
        """

        if stdin is not None:
            sys.stdin = StringIO(stdin)

        with OutputCapture() as output:
            try:
                self.module.main()
            except SystemExit as e:
                self.retcode = e.code

        self.stdout = output.stdout
        self.stderr = output.stderr

        sys.stdin = sys.__stdin__  # restore

        return

    def test_default(self):
        """$ pdb_tidy data/dummy.pdb"""

        fpath = os.path.join(data_dir, 'dummy.pdb')
        sys.argv = ['', fpath]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        # CONECTs are ignored by issue #72, expected only 205 lines
        self.assertEqual(len(self.stdout), 205)
        self.assertEqual(len(self.stderr), 0)  # no errors

        # Check if we added TER statements correctly
        n_ter = len([r for r in self.stdout if r.startswith('TER')])
        self.assertEqual(n_ter, 5)

        # Check no CONECT in output
        c_conect = sum(1 for i in self.stdout if i.startswith('CONECT'))
        self.assertEqual(c_conect, 0)

        # Check if we added END statements correctly
        self.assertTrue(self.stdout[-1].startswith('END'))

    def test_default_in_lib(self):
        """
        Test command-line versus lib.

        $ pdb_tidy data/dummy.pdb

        >>> original_lines = open('dummy.pdb').readlines()
        >>> lines = list(pdb_tidy.run(original_lines))

        `lines` and the result from the command-line must be the same.
        """

        fpath = os.path.join(data_dir, 'dummy.pdb')
        sys.argv = ['', fpath]

        # Execute the script
        self.exec_module()

        fin = open(os.path.join(data_dir, 'dummy.pdb'))
        original_lines = fin.readlines()
        fin.close()
        lines = list(self.module.run(original_lines))

        self.assertEqual(len(lines), 205)
        # Check if we added TER statements correctly
        n_ter = len([r for r in lines if r.startswith('TER')])
        self.assertEqual(n_ter, 5)

        # Check no CONECT in output
        c_conect = sum(1 for i in lines if i.startswith('CONECT'))
        self.assertEqual(c_conect, 0)

        # Check if we added END statements correctly
        self.assertTrue(lines[-1].startswith('END'))

        self.assertTrue(lines, self.stdout)

    def test_tidy_removes_master(self):
        """Test pdb_tidy removes MASTER lines as well."""
        sys.argv = ['']
        self.exec_module(
            'MASTER      378    0    9    5    3    '
            '0   29    6  847    1   76    7'
            )

        # Check no MASTER in output
        m_master = sum(1 for i in self.stdout if i.startswith('MASTER'))
        self.assertEqual(m_master, 0)

    def test_tidy_rewraps_long_lines(self):
        """Test pdb_tidy re-wraps long lines in the header maintaining padding"""
        sys.argv = ['']
        self.exec_module(
            'TITLE     THIS IS A VERY LONG LINE WHICH SHOULD BE WRAPPED '
            'CORRECTLY OVER MULTIPLE LINES WITH PADDING'
        )

        # Check no long lines in output
        long_lines = [line for line in self.stdout if len(line) > 80]
        self.assertEqual(len(long_lines), 0)

        # Check correct wrapping and padding
        self.assertEqual(self.stdout[0],
                         'TITLE     THIS IS A VERY LONG LINE WHICH SHOULD BE '
                         'WRAPPED CORRECTLY OVER       ')
        self.assertEqual(self.stdout[1],
                         'TITLE     MULTIPLE LINES WITH PADDING              '
                         '                             ')

    def test_default_stdin(self):
        """$ cat data/dummy.pdb | pdb_tidy"""

        fpath = os.path.join(data_dir, 'dummy.pdb')
        sys.argv = ['']

        # Execute the script with file as stdin
        with open(fpath) as fp:
            self.exec_module(fp.read())

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        # CONECTs are ignored by issue #72, expected only 205 lines
        self.assertEqual(len(self.stdout), 205)
        self.assertEqual(len(self.stderr), 0)  # no errors

        # Check if we added TER statements correctly
        n_ter = len([r for r in self.stdout if r.startswith('TER')])
        self.assertEqual(n_ter, 5)

        # Check no CONECT in output
        c_conect = sum(1 for i in self.stdout if i.startswith('CONECT'))
        self.assertEqual(c_conect, 0)

        # Check if we added END statements correctly
        self.assertTrue(self.stdout[-1].startswith('END'))

    def test_default_strict(self):
        """$ pdb_tidy -strict data/dummy.pdb"""

        fpath = os.path.join(data_dir, 'dummy.pdb')
        sys.argv = ['', '-strict', fpath]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        # CONECTs are ignored by issue #72, expected only 204 lines
        self.assertEqual(len(self.stdout), 204)
        self.assertEqual(len(self.stderr), 0)  # no errors

        # Check if we added TER statements correctly
        n_ter = len([r for r in self.stdout if r.startswith('TER')])
        self.assertEqual(n_ter, 4)

        # Check no CONECT in output
        c_conect = sum(1 for i in self.stdout if i.startswith('CONECT'))
        self.assertEqual(c_conect, 0)

        # Check if we added END statements correctly
        self.assertTrue(self.stdout[-1].startswith('END'))

    def test_default_strict_stdin(self):
        """$ cat data/dummy.pdb | pdb_tidy -strict"""

        fpath = os.path.join(data_dir, 'dummy.pdb')
        sys.argv = ['', '-strict']

        # Execute the script
        with open(fpath) as fp:
            self.exec_module(fp.read())

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        # CONECTs are ignored by issue #72, expected only 204 lines
        self.assertEqual(len(self.stdout), 204)
        self.assertEqual(len(self.stderr), 0)  # no errors

        # Check if we added TER statements correctly
        n_ter = len([r for r in self.stdout if r.startswith('TER')])
        self.assertEqual(n_ter, 4)

        # Check no CONECT in output
        c_conect = sum(1 for i in self.stdout if i.startswith('CONECT'))
        self.assertEqual(c_conect, 0)

        # Check if we added END statements correctly
        self.assertTrue(self.stdout[-1].startswith('END'))

    def test_corrects_model_format_and_numbers(self):
        """Correct MODEL lines."""
        fpath = os.path.join(data_dir, 'ensemble_error_MODEL.pdb')
        sys.argv = ['', fpath]
        self.exec_module()
        self.assertEqual(self.retcode, 0)
        self.assertEqual(len(self.stdout), 16)
        self.assertEqual(len(self.stderr), 0)
        self.assertEqual(len(self.stdout[5].strip()), 14)
        self.assertEqual(len(self.stdout[5]), 80)
        self.assertEqual(len(self.stdout[10].strip()), 14)
        self.assertEqual(len(self.stdout[10]), 80)
        self.assertEqual(self.stdout[5].strip(), "MODEL        1")
        self.assertEqual(self.stdout[10].strip(), "MODEL        2")

    def test_corrects_model_ENDMDL(self):
        """Correct MODEL lines."""
        fpath = os.path.join(data_dir, 'ensemble_error_4.pdb')
        sys.argv = ['', fpath]
        self.exec_module()
        self.assertEqual(self.retcode, 0)
        self.assertEqual(len(self.stdout), 14)
        self.assertEqual(len(self.stderr), 0)

        # MODEL lines
        self.assertEqual(len(self.stdout[3].strip()), 14)
        self.assertEqual(len(self.stdout[3]), 80)
        self.assertEqual(len(self.stdout[3].strip()), 14)
        self.assertEqual(len(self.stdout[8]), 80)
        self.assertEqual(self.stdout[3].strip(), "MODEL        1")
        self.assertEqual(self.stdout[8].strip(), "MODEL        2")

        # ENDMDL LINES
        self.assertEqual(len(self.stdout[7]), 80)
        self.assertEqual(len(self.stdout[12]), 80)
        self.assertEqual(self.stdout[7].strip(), "ENDMDL")
        self.assertEqual(self.stdout[12].strip(), "ENDMDL")

        # END LINES
        self.assertEqual(self.stdout[-1].strip(), "END")
        self.assertEqual(len(self.stdout[-1]), 80)

    def test_corrects_model_ENDMDL_with_HETATM(self):
        """Correct MODEL lines."""
        fpath = os.path.join(data_dir, 'hetatm_ensemble.pdb')
        sys.argv = ['', fpath]
        self.exec_module()
        self.assertEqual(self.retcode, 0)
        self.assertEqual(len(self.stdout), 45)
        self.assertEqual(len(self.stderr), 0)

        # MODEL lines
        self.assertEqual(len(self.stdout[0]), 80)
        self.assertEqual(len(self.stdout[0].strip()), 14)
        self.assertEqual(len(self.stdout[22]), 80)
        self.assertEqual(len(self.stdout[22].strip()), 14)
        self.assertEqual(self.stdout[0].strip(), "MODEL        1")
        self.assertEqual(self.stdout[22].strip(), "MODEL        2")

        # ENDMDL LINES
        self.assertEqual(len(self.stdout[21]), 80)
        self.assertEqual(len(self.stdout[43]), 80)
        self.assertEqual(self.stdout[21].strip(), "ENDMDL")
        self.assertEqual(self.stdout[43].strip(), "ENDMDL")

        # END LINES
        self.assertEqual(self.stdout[-1].strip(), "END")
        self.assertEqual(len(self.stdout[-1]), 80)


    def test_file_not_found(self):
        """$ pdb_tidy not_existing.pdb"""

        # Error (file not found)
        afile = os.path.join(data_dir, 'not_existing.pdb')
        sys.argv = ['', afile]

        # Execute the script
        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:22],
                         "ERROR!! File not found")

    @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    def test_helptext(self):
        """$ pdb_tidy"""

        sys.argv = ['']

        # Execute the script
        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr, self.module.__doc__.split("\n")[:-1])

    def test_invalid_option(self):
        """$ pdb_tidy -A data/dummy.pdb"""

        sys.argv = ['', '-A', os.path.join(data_dir, 'dummy.pdb')]

        # Execute the script
        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:36],
                         "ERROR! First argument is not a valid")

    def test_noh36_option(self):
        """$ pdb_tidy data/dummy-h36.pdb"""

        sys.argv = ['', os.path.join(data_dir, 'dummy-h36.pdb')]

        # Execute the script
        self.exec_module()

        self.assertEqual(self.stderr[0][:49],
                         "ERROR!! Structure contains more than 99.999 atoms")

    def test_h36_option(self):
        """$ pdb_tidy -h36 data/dummy-h36.pdb"""

        sys.argv = ['', '-h36', os.path.join(data_dir, 'dummy-h36.pdb')]

        # Execute the script
        self.exec_module()

        self.assertEqual(len(self.stdout), 29)
        self.assertIn("A0NN2", self.stdout[26])

if __name__ == '__main__':
    from config import test_dir

    mpath = os.path.abspath(os.path.join(test_dir, '..'))
    sys.path.insert(0, mpath)  # so we load dev files before  any installation

    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 1118 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit Tests for `pdb_reatom`.
"""

import os
import sys
import unittest

from config import data_dir
from utils import OutputCapture


class TestTool(unittest.TestCase):
    """
    Generic class for testing tools.
    """

    def setUp(self):
        # Dynamically import the module
        name = 'pdbtools.pdb_reatom'
        self.module = __import__(name, fromlist=[''])

    def exec_module(self):
        """
        Execs module.
        """

        with OutputCapture() as output:
            try:
                self.module.main()
            except SystemExit as e:
                self.retcode = e.code

        self.stdout = output.stdout
        self.stderr = output.stderr

        return

    def test_default(self):
        """$ pdb_reatom data/dummy.pdb"""

        # Simulate input
        # pdb_reatom dummy.pdb
        sys.argv = ['', os.path.join(data_dir, 'dummy.pdb')]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        self.assertEqual(len(self.stdout), 204)  # no lines deleted
        self.assertEqual(len(self.stderr), 0)  # no errors

        records = (('ATOM', 'HETATM'))
        serial_list = [int(l[6:11]) for l in self.stdout
                       if l.startswith(records)]

        expected = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
                    18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32,
                    33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47,
                    48, 49, 50, 51, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63,
                    64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78,
                    79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93,
                    94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106,
                    107, 108, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119,
                    120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131,
                    132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143,
                    144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155,
                    156, 157, 158, 160, 161, 162, 163, 164, 165, 166, 167, 168,
                    169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 180,
                    181, 182, 183, 184, 185, 186, 187, 188]

        self.assertEqual(serial_list, expected)

    def test_two_options_pos(self):
        """$ pdb_reatom -10 data/dummy.pdb"""

        # Simulate input
        # pdb_reatom dummy.pdb
        sys.argv = ['', '-10', os.path.join(data_dir, 'dummy.pdb')]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        self.assertEqual(len(self.stdout), 204)  # no lines deleted
        self.assertEqual(len(self.stderr), 0)  # no errors

        records = (('ATOM', 'HETATM'))
        serial_list = [int(l[6:11]) for l in self.stdout
                       if l.startswith(records)]

        expected = [10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24,
                    25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
                    40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54,
                    55, 56, 57, 58, 59, 60, 62, 63, 64, 65, 66, 67, 68, 69, 70,
                    71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85,
                    86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100,
                    101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112,
                    113, 114, 115, 116, 117, 119, 120, 121, 122, 123, 124, 125,
                    126, 127, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137,
                    138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 149,
                    150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160, 161,
                    162, 163, 164, 165, 166, 167, 169, 170, 171, 172, 173, 174,
                    175, 176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186,
                    187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197]

        self.assertEqual(serial_list, expected)

    def test_two_options_neg(self):
        """$ pdb_reatom --10 data/dummy.pdb"""

        sys.argv = ['', '--10', os.path.join(data_dir, 'dummy.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 0)
        self.assertEqual(len(self.stdout), 204)
        self.assertEqual(len(self.stderr), 0)

        records = (('ATOM', 'HETATM'))
        serial_list = [int(l[6:11]) for l in self.stdout
                       if l.startswith(records)]

        expected = [-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5,
                    6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21,
                    22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36,
                    37, 38, 39, 40, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52,
                    53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67,
                    68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82,
                    83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97,
                    99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110,
                    111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122,
                    123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 133, 134,
                    135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146,
                    147, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159,
                    160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171,
                    172, 173, 174, 175, 176, 177]

        self.assertEqual(serial_list, expected)

    def test_too_many_atoms(self):
        """$ pdb_reatom -99998 data/dummy.pdb"""

        sys.argv = ['', '-99998', os.path.join(data_dir, 'dummy.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(len(self.stderr), 19)

        self.assertEqual(self.stderr[0][:31],
                         "ERROR!! Structure contains more")  # proper error message

    def test_file_not_found(self):
        """$ pdb_reatom -10 not_existing.pdb"""

        afile = os.path.join(data_dir, 'not_existing.pdb')
        sys.argv = ['', '-10', afile]

        self.exec_module()

        self.assertEqual(self.retcode, 1)  # exit code is 1 (error)
        self.assertEqual(len(self.stdout), 0)  # nothing written to stdout
        self.assertEqual(self.stderr[0][:22],
                         "ERROR!! File not found")  # proper error message

    @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    def test_file_missing(self):
        """$ pdb_reatom -10"""

        sys.argv = ['', '-10']

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)  # no output
        self.assertEqual(self.stderr[0],
                         "ERROR!! No data to process!")

    @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    def test_helptext(self):
        """$ pdb_reatom"""

        sys.argv = ['']

        self.exec_module()

        self.assertEqual(self.retcode, 1)  # ensure the program exited gracefully.
        self.assertEqual(len(self.stdout), 0)  # no output
        self.assertEqual(self.stderr, self.module.__doc__.split("\n")[:-1])

    def test_invalid_option(self):
        """$ pdb_reatom -A data/dummy.pdb"""

        sys.argv = ['', '-A', os.path.join(data_dir, 'dummy.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:47],
                         "ERROR!! You provided an invalid atom serial num")

    def test_not_an_option(self):
        """$ pdb_reatom 11 data/dummy.pdb"""

        sys.argv = ['', '11', os.path.join(data_dir, 'dummy.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0],
                         "ERROR! First argument is not an option: '11'")

    def test_h36_option(self):
        """$ pdb_reatom -h36 data/dummy-h36.pdb"""

        sys.argv = ['', '-h36', os.path.join(data_dir, 'dummy-h36.pdb')]

        # Execute the script
        self.exec_module()

        self.assertEqual(len(self.stdout), 28)
        self.assertIn("27  O   ALA C6913", self.stdout[26])

    def test_h36_100000_option(self):
        """$ pdb_reatom -100000 -h36 data/dummy-h36.pdb"""

        sys.argv = ['', '-100000', '-h36', os.path.join(data_dir, 'dummy-h36.pdb')]

        # Execute the script
        self.exec_module()

        self.assertEqual(len(self.stdout), 28)
        self.assertIn("A000Q", self.stdout[26])


if __name__ == '__main__':
    from config import test_dir

    mpath = os.path.abspath(os.path.join(test_dir, '..'))
    sys.path.insert(0, mpath)  # so we load dev files before  any installation

    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 1118 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit Tests for `pdb_splitchain`.
"""

import os
import shutil
import sys
import tempfile
import unittest

from config import data_dir
from utils import OutputCapture


class TestTool(unittest.TestCase):
    """
    Generic class for testing tools.
    """

    def setUp(self):
        # Dynamically import the module
        name = 'pdbtools.pdb_splitchain'
        self.module = __import__(name, fromlist=[''])
        self.tempdir = tempfile.mkdtemp()  # set temp dir
        os.chdir(self.tempdir)

    def tearDown(self):
        os.chdir(os.path.dirname(os.path.abspath('.')))  # cd ../
        shutil.rmtree(self.tempdir)

    def exec_module(self):
        """
        Execs module.
        """

        with OutputCapture() as output:
            try:
                self.module.main()
            except SystemExit as e:
                self.retcode = e.code

        self.stdout = output.stdout
        self.stderr = output.stderr

        return


    def test_default(self):
        """$ pdb_splitchain data/dummy.pdb"""

        # Copy input file to tempdir

        # Simulate input
        src = os.path.join(data_dir, 'dummy.pdb')
        dst = os.path.join(self.tempdir, 'dummy.pdb')
        shutil.copy(src, dst)
        sys.argv = ['', dst]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        self.assertEqual(len(self.stdout), 0)  # no output
        self.assertEqual(len(self.stderr), 0)  # no errors

        # Read files created by script and then delete
        ofiles = [f for f in os.listdir(self.tempdir) if f.startswith('dummy')]
        self.assertEqual(len(ofiles), 4 + 1)  # ori + 4 chains

        # Make sure each file has the chain it should have
        records = (('ATOM', 'HETATM', 'TER', 'ANISOU'))
        for fpath in ofiles:
            if fpath == 'dummy.pdb':
                continue

            with open(os.path.join(self.tempdir, fpath), 'r') as handle:
                fname_chain = fpath.split('_')[1][:-4]  # xxx_(X).pdb
                pdb_chains = [l[21] for l in handle if l.startswith(records)]

                self.assertEqual(fname_chain, list(set(pdb_chains))[0])

    def test_run_fhandler(self):
        """pdb_splitchain.run(fhandler)"""
        from pdbtools import pdb_splitchain

        src = os.path.join(data_dir, 'dummy.pdb')
        dst = os.path.join(self.tempdir, 'dummy.pdb')
        shutil.copy(src, dst)

        with open(dst, 'r') as fin:
            pdb_splitchain.run(fin)

        # Read files created by script and then delete
        ofiles = [f for f in os.listdir(self.tempdir) if f.startswith('dummy')]
        self.assertEqual(len(ofiles), 4 + 1)  # ori + 4 chains

        # Make sure each file has the chain it should have
        records = (('ATOM', 'HETATM', 'TER', 'ANISOU'))
        for fpath in ofiles:
            if fpath == 'dummy.pdb':
                continue

            with open(os.path.join(self.tempdir, fpath), 'r') as handle:
                fname_chain = fpath.split('_')[1][:-4]  # xxx_(X).pdb
                pdb_chains = [l[21] for l in handle if l.startswith(records)]

                self.assertEqual(fname_chain, list(set(pdb_chains))[0])

    def test_run_iterable(self):
        """pdb_splitchain.run(iterable)"""
        from pdbtools import pdb_splitchain

        src = os.path.join(data_dir, 'dummy.pdb')
        dst = os.path.join(self.tempdir, 'dummy.pdb')
        shutil.copy(src, dst)

        with open(dst, 'r') as fin:
            lines = fin.readlines()

        pdb_splitchain.run(lines)

        # Read files created by script and then delete
        ofiles = [
            f
            for f in os.listdir(self.tempdir)
            if f.startswith('splitchains')
            ]
        self.assertEqual(len(ofiles), 4)  # 4 chains

        # Make sure each file has the chain it should have
        records = (('ATOM', 'HETATM', 'TER', 'ANISOU'))
        for fpath in ofiles:

            with open(os.path.join(self.tempdir, fpath), 'r') as handle:
                fname_chain = fpath.split('_')[1][:-4]  # xxx_(X).pdb
                pdb_chains = [l[21] for l in handle if l.startswith(records)]

                self.assertEqual(fname_chain, list(set(pdb_chains))[0])

    def test_run_iterable_with_name(self):
        """pdb_splitchain.run(iterable)"""
        from pdbtools import pdb_splitchain

        src = os.path.join(data_dir, 'dummy.pdb')
        dst = os.path.join(self.tempdir, 'dummy.pdb')
        shutil.copy(src, dst)

        with open(dst, 'r') as fin:
            lines = fin.readlines()

        pdb_splitchain.run(lines, outname='newname')

        # Read files created by script and then delete
        ofiles = [f for f in os.listdir(self.tempdir) if f.startswith('newname')]
        self.assertEqual(len(ofiles), 4)  # 4 chains

        # Make sure each file has the chain it should have
        records = (('ATOM', 'HETATM', 'TER', 'ANISOU'))
        for fpath in ofiles:

            with open(os.path.join(self.tempdir, fpath), 'r') as handle:
                fname_chain = fpath.split('_')[1][:-4]  # xxx_(X).pdb
                pdb_chains = [l[21] for l in handle if l.startswith(records)]

                self.assertEqual(fname_chain, list(set(pdb_chains))[0])

    def test_file_not_found(self):
        """$ pdb_splitchain not_existing.pdb"""

        afile = os.path.join(data_dir, 'not_existing.pdb')
        sys.argv = ['', afile]

        self.exec_module()

        self.assertEqual(self.retcode, 1)  # exit code is 1 (error)
        self.assertEqual(len(self.stdout), 0)  # nothing written to stdout
        self.assertEqual(self.stderr[0][:22],
                         "ERROR!! File not found")  # proper error message

    @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    def test_file_missing(self):
        """$ pdb_splitchain -10"""

        sys.argv = ['', '-10']

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)  # no output
        self.assertEqual(self.stderr[0][:38],
                         "ERROR!! File not found or not readable")

    @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    def test_helptext(self):
        """$ pdb_splitchain"""

        sys.argv = ['']

        self.exec_module()

        self.assertEqual(self.retcode, 1)  # ensure the program exited gracefully.
        self.assertEqual(len(self.stdout), 0)  # no output
        self.assertEqual(self.stderr, self.module.__doc__.split("\n")[:-1])

    def test_invalid_option(self):
        """$ pdb_splitchain -A data/dummy.pdb"""

        sys.argv = ['', '-A', os.path.join(data_dir, 'dummy.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:22],
                         "ERROR!! Script takes 1")  # proper error message


if __name__ == '__main__':
    from config import test_dir

    mpath = os.path.abspath(os.path.join(test_dir, '..'))
    sys.path.insert(0, mpath)  # so we load dev files before  any installation

    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2022 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit Tests for `pdb_selmodel`.
"""

import os
import sys
import unittest

from config import data_dir
from utils import OutputCapture


class TestTool(unittest.TestCase):
    """
    Generic class for testing tools.
    """

    def setUp(self):
        # Dynamically import the module
        name = 'pdbtools.pdb_selmodel'
        self.module = __import__(name, fromlist=[''])

    def exec_module(self):
        """
        Execs module.
        """

        with OutputCapture() as output:
            try:
                self.module.main()
            except SystemExit as e:
                self.retcode = e.code

        self.stdout = output.stdout
        self.stderr = output.stderr

        return

    def test_one_option(self):
        """$ pdb_selmodel -1 data/ensemble_OK.pdb"""

        # Simulate input
        # pdb_selmodel ensemble_OK.pdb
        sys.argv = ['', '-1', os.path.join(data_dir, 'ensemble_OK.pdb')]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        self.assertEqual(len(self.stdout), 7)  # selected model 1
        self.assertEqual(len(self.stderr), 0)  # no errors

    def test_multiple(self):
        """
        $ pdb_selmodel -1,2 data/ensemble_OK.pdb
        """

        sys.argv = ['', '-1,2', os.path.join(data_dir, 'ensemble_OK.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 0)
        self.assertEqual(len(self.stdout), 11)  # all models
        self.assertEqual(len(self.stderr), 0)

    def test_file_not_found(self):
        """$ pdb_selmodel not_existing.pdb"""

        afile = os.path.join(data_dir, 'not_existing.pdb')
        sys.argv = ['', afile]

        self.exec_module()

        self.assertEqual(self.retcode, 1)  # exit code is 1 (error)
        self.assertEqual(len(self.stdout), 0)  # nothing written to stdout
        self.assertEqual(self.stderr[0][:22],
                         "ERROR!! File not found")  # proper error message

    @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    def test_file_missing(self):
        """$ pdb_selmodel -1"""

        sys.argv = ['', '-1']

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)  # no output
        self.assertEqual(self.stderr[0],
                         "ERROR!! No data to process!")

    @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    def test_helptext(self):
        """$ pdb_selmodel"""

        sys.argv = ['']

        self.exec_module()

        self.assertEqual(self.retcode, 1)  # ensure the program exited gracefully.
        self.assertEqual(len(self.stdout), 0)  # no output
        self.assertEqual(self.stderr, self.module.__doc__.split("\n")[:-1])

    def test_invalid_option(self):
        """$ pdb_selmodel data/ensemble_OK.pdb"""

        sys.argv = ['', os.path.join(data_dir, 'ensemble_OK.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:47],
                         "ERROR!! You must provide at least one model ide")

    def test_invalid_option_2(self):
        """$ pdb_selmodel -AB data/ensemble_OK.pdb"""

        sys.argv = ['', '-AB', os.path.join(data_dir, 'ensemble_OK.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0][:40],
                "ERROR!! Model identifier is invalid: 'AB")

    def test_not_an_option(self):
        """$ pdb_selmodel 20 data/ensemble_OK.pdb"""

        sys.argv = ['', '20', os.path.join(data_dir, 'ensemble_OK.pdb')]

        self.exec_module()

        self.assertEqual(self.retcode, 1)
        self.assertEqual(len(self.stdout), 0)
        self.assertEqual(self.stderr[0],
                         "ERROR! First argument is not an option: '20'")


if __name__ == '__main__':
    from config import test_dir

    mpath = os.path.abspath(os.path.join(test_dir, '..'))
    sys.path.insert(0, mpath)  # so we load dev files before  any installation

    unittest.main()



#!/usr/bin/env python

"""
Scrapes tool headers for documentation.
"""

import importlib
from pathlib import Path

with open('index.md', 'w') as handle:

    with open('index.md.template') as template:
        print(template.read(), file=handle)

    import pdbtools
    modfile = Path(pdbtools.__file__)
    for f in sorted(list(modfile.parent.iterdir())):
        # ignore __init__.py and others.
        if f.stem.startswith('_') or f.suffix != '.py':
            continue

        # Dynamically import tool to get __doc__
        name = f.stem
        try:
            tool = importlib.import_module(f'pdbtools.{name}')
        except ModuleNotFoundError:
            print(f'Could not import module: {name}')
            continue

        # Parse documentation from docstrings
        # Preserve white-space as best as possible.
        # First non-empty line is always short description.
        # Last lines are always licensing disclaimer
        summary = None
        long_description = []

        doctext = tool.__doc__.replace('<', '&lt;').replace('>', '&gt;')
        for line in doctext.split('\n'):
            if summary is None and not line.strip():
                continue
            if line.startswith('This program is part of the'):
                break
            elif summary is None:
                summary = line
            else:
                long_description.append(line)

        long_description = '\n'.join(long_description)
        print('<div style="margin-bottom: 1em;">', file=handle)
        print('<details>', file=handle)
        print(f"<summary><b>{name}</b><p>{summary}</p></summary>", file=handle)
        print(
            f'<span style="font-family: monospace; white-space: pre;">{long_description}</span>',
            file=handle
        )
        print('</details>', file=handle)
        print('</div>', file=handle)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Renames chain identifiers sequentially, based on TER records.

Since HETATM records are not separated by TER records and usually come together
at the end of the PDB file, this script will attempt to reassign their chain
identifiers based on the changes it made to ATOM lines. This might lead to bad
output in certain corner cases.

Usage:
    python pdb_chainbows.py <pdb file>

Example:
    python pdb_chainbows.py 1CTF.pdb

This program is part of the `pdb-tools` suite of utilities and should not be
distributed isolatedly. The `pdb-tools` were created to quickly manipulate PDB
files using the terminal, and can be used sequentially, with one tool streaming
data to another. They are based on old FORTRAN77 code that was taking too much
effort to maintain and compile. RIP.
"""

import os
import string
import sys

__author__ = "Joao Rodrigues"
__email__ = "j.p.g.l.m.rodrigues@gmail.com"


def check_input(args):
    """Checks whether to read from stdin/file and validates user input/options.
    """

    # Defaults
    fh = sys.stdin  # file handle

    if not len(args):
        # Reading from pipe with default option
        if sys.stdin.isatty():
            sys.stderr.write(__doc__)
            sys.exit(1)

    elif len(args) == 1:
        if not os.path.isfile(args[0]):
            emsg = 'ERROR!! File not found or not readable: \'{}\'\n'
            sys.stderr.write(emsg.format(args[0]))
            sys.stderr.write(__doc__)
            sys.exit(1)

        fh = open(args[0], 'r')

    else:  # Whatever ...
        emsg = 'ERROR!! Script takes 1 argument, not \'{}\'\n'
        sys.stderr.write(emsg.format(len(args)))
        sys.stderr.write(__doc__)
        sys.exit(1)

    return fh


def run(fhandle):
    """
    Set chains sequentially based on existing TER records.

    Follow sequence [ABC...abc...012...].

    This function is a generator.

    Parameters
    ----------
    fhandle : an iterable giving the PDB file line-by-line

    Yields
    ------
    str (line-by-line)
        The modified (or not) PDB line.
    """
    chainlist = list(
        string.digits[::-1] + string.ascii_lowercase[::-1] + string.ascii_uppercase[::-1]
    )  # 987...zyx...cbaZYX...BCA.
    max_chains = len(chainlist)

    chain_map = {}  # for HETATM.

    curchain = chainlist.pop()
    records = ('ATOM', 'TER', 'ANISOU')
    for line in fhandle:
        if line.startswith(records):
            chain_map[line[21]] = curchain
            line = line[:21] + curchain + line[22:]

            if line.startswith('TER'):
                try:
                    curchain = chainlist.pop()
                except IndexError:
                    emsg = 'ERROR!! Structure contains more than {} TER records.\n'
                    sys.stderr.write(emsg.format(max_chains))
                    sys.stderr.write(__doc__)
                    sys.exit(1)

        elif line.startswith('HETATM'):
            hetchain = chain_map[line[21]]
            line = line[:21] + hetchain + line[22:]

        yield line


set_chain_sequence = run


def main():
    # Check Input
    pdbfh = check_input(sys.argv[1:])

    # Do the job
    new_pdb = run(pdbfh)

    try:
        _buffer = []
        _buffer_size = 5000  # write N lines at a time
        for lineno, line in enumerate(new_pdb):
            if not (lineno % _buffer_size):
                sys.stdout.write(''.join(_buffer))
                _buffer = []
            _buffer.append(line)

        sys.stdout.write(''.join(_buffer))
        sys.stdout.flush()
    except IOError:
        # This is here to catch Broken Pipes
        # for example to use 'head' or 'tail' without
        # the error message showing up
        pass

    # last line of the script
    # We can close it even if it is sys.stdin
    pdbfh.close()
    sys.exit(0)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2018 João Pedro Rodrigues
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit Tests for `pdb_intersect`.
"""

import os
import sys
import unittest

from config import data_dir
from utils import OutputCapture


class TestTool(unittest.TestCase):
    """
    Generic class for testing tools.
    """

    def setUp(self):
        # Dynamically import the module
        name = 'pdbtools.pdb_intersect'
        self.module = __import__(name, fromlist=[''])

    def exec_module(self):
        """
        Execs module.
        """

        with OutputCapture() as output:
            try:
                self.module.main()
            except SystemExit as e:
                self.retcode = e.code

        self.stdout = output.stdout
        self.stderr = output.stderr

        return

    def test_default(self):
        """$ pdb_intersect data/dummy.pdb data/dummy.pdb"""

        # Simulate input
        sys.argv = ['', os.path.join(data_dir, 'dummy.pdb'),
                    os.path.join(data_dir, 'dummy.pdb')]

        # Execute the script
        self.exec_module()

        # Validate results
        self.assertEqual(self.retcode, 0)  # ensure the program exited OK.
        self.assertEqual(len(self.stdout), 188)  # no lines deleted. Same file.
        self.assertEqual(len(self.stderr), 0)  # no errors

        atom_names = [l[12:16] for l in self.stdout]

        # Test content
        atoms_list = [' N  ', ' H  ', ' H2 ', ' H3 ', ' CA ', ' HA ', ' CB ',
                      ' HB2', ' HB3', ' CG ', ' HG2', ' HG3', ' CD ', ' HD2',
                      ' HD3', ' NE ', ' HE ', ' CZ ', ' NH1', 'HH11', 'HH12',
                      ' NH2', 'HH21', 'HH22', ' C  ', ' O  ', ' N  ', ' H  ',
                      ' CA ', ' HA ', ' CB ', ' HB2', ' HB3', ' CG ', ' HG2',
                      ' HG3', ' CD ', ' OE1', ' OE2', ' C  ', ' O  ', ' N  ',
                      ' H  ', ' CA ', ' HA ', ' CB ', ' HB1', ' HB2', ' HB3',
                      ' C  ', ' O  ', '    ', ' N  ', ' H  ', ' H2 ', ' H3 ',
                      ' CA ', ' CA ', ' HA ', ' CB ', ' HB2', ' HB3', ' CG ',
                      ' OD1', ' ND2', 'HD21', 'HD22', ' C  ', ' O  ', ' N  ',
                      ' H  ', ' CA ', ' HA ', ' CB ', ' HB2', ' HB3', ' CG ',
                      ' HG2', ' HG3', ' CD ', ' HD2', ' HD3', ' NE ', ' HE ',
                      ' CZ ', ' NH1', 'HH11', 'HH12', ' NH2', 'HH21', 'HH22',
                      ' C  ', ' O  ', ' N  ', ' H  ', ' CA ', ' HA ', ' CB ',
                      ' HB2', ' HB3', ' CG ', ' HG2', ' HG3', ' CD ', ' OE1',
                      ' OE2', ' C  ', ' O  ', '    ', ' N  ', ' H  ', ' H2 ',
                      ' H3 ', ' CA ', ' HA ', ' CB ', ' HB2', ' HB3', ' CG ',
                      ' HG2', ' HG3', ' CD ', ' HD2', ' HD3', ' NE ', ' HE ',
                      ' CZ ', ' NH1', 'HH11', 'HH12', ' NH2', 'HH21', 'HH22',
                      ' C  ', ' O  ', ' N  ', ' H  ', ' CA ', ' HA ', ' CB ',
                      ' HB2', ' HB3', ' CG ', ' HG2', ' HG3', ' CD ', ' OE1',
                      ' OE2', ' C  ', ' O  ', ' N  ', ' CA ', ' C  ', ' O  ',
                      ' CB ', ' CG ', ' SD ', ' CE ', '    ', ' P  ', ' OP1',
                      ' OP2', " O5'", " C5'", " C4'", " O4'", " C3'", " O3'",
                      " C2'", " C1'", ' N1 ', ' C2 ', ' O2 ', ' N3 ', ' C4 ',
                      ' O4 ', ' C5 ', ' C7 ', ' C6 ', 'CA  ', ' O  ', ' O  ',
                      ' O  ', ' O  ', ' O  ', ' O  ', ' O  ', ' O  ']

        self.assertEqual(atoms_list, atom_names)

    def test_file_not_found(self):
        """$ pdb_intersect not_existing.pdb"""

        # Error (file not found)
        afile = os.path.join(data_dir, 'not_existing.pdb')
        sys.argv = ['', afile]

        # Execute the script
        self.exec_module()

        self.assertEqual(self.retcode, 1)  # exit code is 1 (error)
        self.assertEqual(len(self.stdout), 0)  # nothing written to stdout
        self.assertEqual(self.stderr[0][:22],
                         "ERROR!! File not found")  # proper error message

    @unittest.skipIf(os.getenv('SKIP_TTY_TESTS'), 'skip on GHA - no TTY')
    def test_helptext(self):
        """$ pdb_intersect"""

        sys.argv = ['']

        # Execute the script
        self.exec_module()

        self.assertEqual(self.retcode, 1)  # ensure the program exited gracefully.
        self.assertEqual(len(self.stdout), 0)  # no output
        self.assertEqual(self.stderr, self.module.__doc__.split("\n")[:-1])


if __name__ == '__main__':
    from config import test_dir

    mpath = os.path.abspath(os.path.join(test_dir, '..'))
    sys.path.insert(0, mpath)  # so we load dev files before  any installation

    unittest.main()



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from os import listdir, path

# io.open is needed for projects that support Python 2.7
# It ensures open() defaults to text mode with universal newlines,
# and accepts an argument to specify the text encoding
# Python 3 only projects can skip this import
from io import open

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# Collect names of bin/*py scripts
# e.g. 'pdb_intersect=bin.pdb_intersect:main',
binfiles = listdir(path.join(here, "pdbtools"))
bin_py = [
    f[:-3] + "=pdbtools." + f[:-3] + ":main" for f in binfiles if f.endswith(".py")
]

setup(
    name="pdb-tools",  # Required
    version="2.7.0",  # Required
    description="A swiss army knife for PDB files.",  # Optional
    long_description=long_description,  # Optional
    long_description_content_type="text/markdown",  # Optional (see note above)
    url="http://bonvinlab.org/pdb-tools",  # Optional
    author="Joao Rodrigues",  # Optional
    author_email="j.p.g.l.m.rodrigues@gmail.com",  # Optional
    license="Apache Software License, version 2",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Chemistry",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3.14",
    ],
    keywords="bioinformatics protein structural-biology pdb",  # Optional
    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    packages=find_packages(),  # Required
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # `pip` to create the appropriate form of executable for the target
    # platform.
    #
    # For example, the following would provide a command called `sample` which
    # executes the function `main` from this package when invoked:
    entry_points={  # Optional
        "console_scripts": bin_py,
    },
    # scripts=[path.join('bin', f) for f in listdir(path.join(here, 'bin'))],
    # List additional URLs that are relevant to your project as a dict.
    #
    # This field corresponds to the "Project-URL" metadata fields:
    # https://packaging.python.org/specifications/core-metadata/#project-url-multiple-use
    #
    # Examples listed include a pattern for specifying where the package tracks
    # issues, where the source is hosted, where to say thanks to the package
    # maintainers, and where to support the project financially. The key is
    # what's used to render the link text on PyPI.
    project_urls={  # Optional
        "Bug Reports": "https://github.com/haddocking/pdb-tools/issues",
        "Source": "https://github.com/haddocking/pdb-tools",
    },
    # Test Suite
    test_suite="tests.test_all",
)
